/*******************************************************************************
* File Name: ECG_Out.c  
* Version 1.90
*
* Description:
*  This file contains API to enable firmware control of a Pins component.
*
* Note:
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "cytypes.h"
#include "ECG_Out.h"

#define SetP4PinDriveMode(shift, mode)  \
    do { \
        ECG_Out_PC =   (ECG_Out_PC & \
                                (uint32)(~(uint32)(ECG_Out_DRIVE_MODE_IND_MASK << (ECG_Out_DRIVE_MODE_BITS * (shift))))) | \
                                (uint32)((uint32)(mode) << (ECG_Out_DRIVE_MODE_BITS * (shift))); \
    } while (0)


/*******************************************************************************
* Function Name: ECG_Out_Write
********************************************************************************
*
* Summary:
*  Assign a new value to the digital port's data output register.  
*
* Parameters:  
*  prtValue:  The value to be assigned to the Digital Port. 
*
* Return: 
*  None 
*  
*******************************************************************************/
void ECG_Out_Write(uint8 value) 
{
    uint8 drVal = (uint8)(ECG_Out_DR & (uint8)(~ECG_Out_MASK));
    drVal = (drVal | ((uint8)(value << ECG_Out_SHIFT) & ECG_Out_MASK));
    ECG_Out_DR = (uint32)drVal;
}


/*******************************************************************************
* Function Name: ECG_Out_SetDriveMode
********************************************************************************
*
* Summary:
*  Change the drive mode on the pins of the port.
* 
* Parameters:  
*  mode:  Change the pins to this drive mode.
*
* Return: 
*  None
*
*******************************************************************************/
void ECG_Out_SetDriveMode(uint8 mode) 
{
	SetP4PinDriveMode(ECG_Out__0__SHIFT, mode);
}


/*******************************************************************************
* Function Name: ECG_Out_Read
********************************************************************************
*
* Summary:
*  Read the current value on the pins of the Digital Port in right justified 
*  form.
*
* Parameters:  
*  None 
*
* Return: 
*  Returns the current value of the Digital Port as a right justified number
*  
* Note:
*  Macro ECG_Out_ReadPS calls this function. 
*  
*******************************************************************************/
uint8 ECG_Out_Read(void) 
{
    return (uint8)((ECG_Out_PS & ECG_Out_MASK) >> ECG_Out_SHIFT);
}


/*******************************************************************************
* Function Name: ECG_Out_ReadDataReg
********************************************************************************
*
* Summary:
*  Read the current value assigned to a Digital Port's data output register
*
* Parameters:  
*  None 
*
* Return: 
*  Returns the current value assigned to the Digital Port's data output register
*  
*******************************************************************************/
uint8 ECG_Out_ReadDataReg(void) 
{
    return (uint8)((ECG_Out_DR & ECG_Out_MASK) >> ECG_Out_SHIFT);
}


/* If Interrupts Are Enabled for this Pins component */ 
#if defined(ECG_Out_INTSTAT) 

    /*******************************************************************************
    * Function Name: ECG_Out_ClearInterrupt
    ********************************************************************************
    *
    * Summary:
    *  Clears any active interrupts attached to port and returns the value of the 
    *  interrupt status register.
    *
    * Parameters:  
    *  None 
    *
    * Return: 
    *  Returns the value of the interrupt status register
    *  
    *******************************************************************************/
    uint8 ECG_Out_ClearInterrupt(void) 
    {
		uint8 maskedStatus = (uint8)(ECG_Out_INTSTAT & ECG_Out_MASK);
		ECG_Out_INTSTAT = maskedStatus;
        return maskedStatus >> ECG_Out_SHIFT;
    }

#endif /* If Interrupts Are Enabled for this Pins component */ 


/* [] END OF FILE */
