//*****************************************************************************
//*****************************************************************************
//  FILENAME: SDCard.h
//  Version 0.0
//
//
//  DESCRIPTION: SDCard User Module C Language header file for the
//               PSoC family of devices.  This header file contains only
//               the functions that are should be used by the programmer.
//
//-----------------------------------------------------------------------------
//  The original source for this user module was purchased from
//  Efficient Computer Systems, LLC.
//-----------------------------------------------------------------------------
//
//  Copyright 2003-2006   Efficient Computer Systems, LLC
//  Licensed only for use on any Cypress PSOC Mixed-Signal Controllers.
//  All rights reserved
//
//-----------------------------------------------------------------------------
//
//         Created 12-04-03   By: Lee W. Morin and Herb Winters
//  1.00   Release 04-24-06   By: Lee Morin, Herb Winters, Eric Curtis
//
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//  Copyright (c) Cypress Semiconductor 2013-14. All Rights Reserved.
//*****************************************************************************
//*****************************************************************************
#if !defined(SDCard_HEADER)
#define SDCard_HEADER

#include "SDCard_SPI.h"
#include "SDCard_FS.h"

#include "SDCard_SD_CS.h"
#include "SDCard_SD_PWR.h"

#include "CyLib.h"

//---------------------------------------------------------------------------------------------
//                                   Function Prototypes
//---------------------------------------------------------------------------------------------

//---------------------------------------------------------------------------------------------
//                                Basic Read Write commands
//---------------------------------------------------------------------------------------------

void    SDCard_Start(void);                       // Starts SD card module
void    SDCard_Stop(void);                        // Stops  SD card module
uint8   SDCard_InitCard(void);                    // Runs all commands to init card for use
uint8   SDCard_fseek(uint8 Fptr, uint32 Offset);  // Seeks a specific offet into file
uint8   SDCard_fgetc(uint8 Fptr);                 // Returns the next character from the file pointed to
uint8   SDCard_fbgetc(uint8 Fptr);                // Returns the next buffered character from the file pointed to
void    SDCard_clearerr(uint8 Fptr);              // Clears the error flags for the file
uint8   SDCard_ferror(uint8 Fptr);                // Returns non-zero for file error, zero if no error
uint32  SDCard_ftell(uint8 Fptr);                 // Return the current postion within the file
uint8   SDCard_ReadSect(uint32 address);          // Read a sector

#ifdef  SDCard_ENABLE_PRESENT
uint8   SDCard_Present(void);                     // Returns a '1' if a card is present in the socket, '0' if not
#endif

#ifdef  SDCard_ENABLE_WPROTECT
uint8   SDCard_WriteProtect(void);                // Returns a '1' if the card is write protected using ther slide switch, '0' if not
#endif

#ifdef ENABLE_WRITE
uint8   SDCard_WriteSect(uint32 address);         // Write a sector
uint8   SDCard_fputc(uint8 Data, uint8 Fptr);     // Write a character to a file
uint8   SDCard_fputs(uint8 *str, uint8 Fptr);     // Writes a null-terminated string to a file.
uint8   SDCard_fputcs(uint8 *str, uint8 Fptr);    // Writes a null-terminated const string to a file.
uint8   SDCard_fputBuff(uint8 *buff, uint16 count, uint8 Fptr);
uint8   SDCard_fputcBuff(uint8 *buff, uint16 count, uint8 Fptr);
void    SDCard_fflush(uint8 Fptr);                // Flush the write buffers (to the file and update dir values in FILESYSTEM mode)
#endif

//---------------------------
// Top level file functions
//---------------------------
#ifdef ENABLE_FILESYSTEM
uint8   SDCard_fclose(uint8 Fptr);                // Closes a file pointer
uint8   SDCard_fopen(uint8 Filename[], uint8 Mode[]);   // Opens a file as type and returns the file pointer
uint8 * SDCard_GetFilename(uint16 Entry);         // Returns filename for directory entry specified.
uint16  SDCard_GetFileCount(void);                // Returns the number of valid files in the root directory
uint32  SDCard_GetFileSize(uint8 Fptr);           // Returns the file size of the file pointed to
uint8   SDCard_feof(uint8 Fptr);                  // Returns non-zero for EOF, zero if no EOF

// ============================================================================
// = Top level writing functions
// ============================================================================
#ifdef ENABLE_WRITE
uint8   SDCard_Remove(uint8 * Filename);          // Delete the file indicated by Filename
uint8   SDCard_Rename(uint8 * OldFilename, uint8 * NewFilename);  // Rename a file
uint8   SDCard_Copy(uint8 * OldFilename, uint8 * NewFilename);    // Copy a file
#endif  // End ENABLE_WRITE
#endif  // End ENABLE_FILESYSTEM

// = CardInfo defines
#define SDCard_FORMAT_FAT12    0x10
#define SDCard_FORMAT_FAT16    0x40
#define SDCard_FORMAT_FAT16a   0x40
#define SDCard_FORMAT_FAT16b   0x60
#define SDCard_FORMAT_FAT32    0xB0
#define SDCard_FORMAT_VALID    0xE0
#define SDCard_FORMAT_MASK     0xF0

#define SDCard_TYPE_NONE       0x00
#define SDCard_TYPE_MMC        0x01
#define SDCard_TYPE_SD         0x02
#define SDCard_TYPE_MASK       0x03

#endif

// ============================================================================

