/*******************************************************************************
* File Name: SDCard_SPIM_PM.c
* Version 1.20
*
* Description:
*  This file provides the source code to the Power Management support for
*  the SCB Component.
*
* Note:
*
********************************************************************************
* Copyright 2013-2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "SDCard_SPIM.h"
#include "SDCard_SPIM_PVT.h"

#if(SDCard_SPIM_SCB_MODE_I2C_INC)
    #include "SDCard_SPIM_I2C_PVT.h"
#endif /* (SDCard_SPIM_SCB_MODE_I2C_INC) */

#if(SDCard_SPIM_SCB_MODE_EZI2C_INC)
    #include "SDCard_SPIM_EZI2C_PVT.h"
#endif /* (SDCard_SPIM_SCB_MODE_EZI2C_INC) */

#if(SDCard_SPIM_SCB_MODE_SPI_INC || SDCard_SPIM_SCB_MODE_UART_INC)
    #include "SDCard_SPIM_SPI_UART_PVT.h"
#endif /* (SDCard_SPIM_SCB_MODE_SPI_INC || SDCard_SPIM_SCB_MODE_UART_INC) */


/***************************************
*   Backup Structure declaration
***************************************/

#if(SDCard_SPIM_SCB_MODE_UNCONFIG_CONST_CFG || \
   (SDCard_SPIM_SCB_MODE_I2C_CONST_CFG   && (!SDCard_SPIM_I2C_WAKE_ENABLE_CONST))   || \
   (SDCard_SPIM_SCB_MODE_EZI2C_CONST_CFG && (!SDCard_SPIM_EZI2C_WAKE_ENABLE_CONST)) || \
   (SDCard_SPIM_SCB_MODE_SPI_CONST_CFG   && (!SDCard_SPIM_SPI_WAKE_ENABLE_CONST))   || \
   (SDCard_SPIM_SCB_MODE_UART_CONST_CFG  && (!SDCard_SPIM_UART_WAKE_ENABLE_CONST)))

    SDCard_SPIM_BACKUP_STRUCT SDCard_SPIM_backup =
    {
        0u, /* enableState */
    };
#endif


/*******************************************************************************
* Function Name: SDCard_SPIM_Sleep
********************************************************************************
*
* Summary:
*  Prepares the component to enter Deep Sleep.
*  The “Enable wakeup from Sleep Mode” selection has an influence on
*  this function implementation.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void SDCard_SPIM_Sleep(void)
{
#if(SDCard_SPIM_SCB_MODE_UNCONFIG_CONST_CFG)

    if(SDCard_SPIM_SCB_WAKE_ENABLE_CHECK)
    {
        if(SDCard_SPIM_SCB_MODE_I2C_RUNTM_CFG)
        {
            SDCard_SPIM_I2CSaveConfig();
        }
        else if(SDCard_SPIM_SCB_MODE_EZI2C_RUNTM_CFG)
        {
            SDCard_SPIM_EzI2CSaveConfig();
        }
    #if(!SDCard_SPIM_CY_SCBIP_V1_I2C_ONLY)
        else if(SDCard_SPIM_SCB_MODE_SPI_RUNTM_CFG)
        {
            SDCard_SPIM_SpiSaveConfig();
        }
        else if(SDCard_SPIM_SCB_MODE_UART_RUNTM_CFG)
        {
            SDCard_SPIM_UartSaveConfig();
        }
    #endif /* (!SDCard_SPIM_CY_SCBIP_V1_I2C_ONLY) */
        else
        {
            /* Unknown mode: do nothing */
        }
    }
    else
    {
        SDCard_SPIM_backup.enableState = (uint8) SDCard_SPIM_GET_CTRL_ENABLED;

        if(0u != SDCard_SPIM_backup.enableState)
        {
            SDCard_SPIM_Stop();
        }
    }

#else

    #if (SDCard_SPIM_SCB_MODE_I2C_CONST_CFG && SDCard_SPIM_I2C_WAKE_ENABLE_CONST)
        SDCard_SPIM_I2CSaveConfig();

    #elif (SDCard_SPIM_SCB_MODE_EZI2C_CONST_CFG && SDCard_SPIM_EZI2C_WAKE_ENABLE_CONST)
        SDCard_SPIM_EzI2CSaveConfig();

    #elif (SDCard_SPIM_SCB_MODE_SPI_CONST_CFG && SDCard_SPIM_SPI_WAKE_ENABLE_CONST)
        SDCard_SPIM_SpiSaveConfig();

    #elif (SDCard_SPIM_SCB_MODE_UART_CONST_CFG && SDCard_SPIM_UART_WAKE_ENABLE_CONST)
        SDCard_SPIM_UartSaveConfig();

    #else

        SDCard_SPIM_backup.enableState = (uint8) SDCard_SPIM_GET_CTRL_ENABLED;

        if(0u != SDCard_SPIM_backup.enableState)
        {
            SDCard_SPIM_Stop();
        }

    #endif /* defined (SDCard_SPIM_SCB_MODE_I2C_CONST_CFG) && (SDCard_SPIM_I2C_WAKE_ENABLE_CONST) */

#endif /* (SDCard_SPIM_SCB_MODE_UNCONFIG_CONST_CFG) */
}


/*******************************************************************************
* Function Name: SDCard_SPIM_Wakeup
********************************************************************************
*
* Summary:
*  Prepares the component for the Active mode operation after exiting Deep Sleep.
*  The “Enable wakeup from Sleep Mode” option has an influence on this function
*  implementation.
*  This function should not be called after exiting Sleep.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void SDCard_SPIM_Wakeup(void)
{
#if(SDCard_SPIM_SCB_MODE_UNCONFIG_CONST_CFG)

    if(SDCard_SPIM_SCB_WAKE_ENABLE_CHECK)
    {
        if(SDCard_SPIM_SCB_MODE_I2C_RUNTM_CFG)
        {
            SDCard_SPIM_I2CRestoreConfig();
        }
        else if(SDCard_SPIM_SCB_MODE_EZI2C_RUNTM_CFG)
        {
            SDCard_SPIM_EzI2CRestoreConfig();
        }
    #if(!SDCard_SPIM_CY_SCBIP_V1_I2C_ONLY)
        else if(SDCard_SPIM_SCB_MODE_SPI_RUNTM_CFG)
        {
            SDCard_SPIM_SpiRestoreConfig();
        }
        else if(SDCard_SPIM_SCB_MODE_UART_RUNTM_CFG)
        {
            SDCard_SPIM_UartRestoreConfig();
        }
    #endif /* (!SDCard_SPIM_CY_SCBIP_V1_I2C_ONLY) */
        else
        {
            /* Unknown mode: do nothing */
        }
    }
    else
    {
        if(0u != SDCard_SPIM_backup.enableState)
        {
            SDCard_SPIM_Enable();
        }
    }

#else

    #if (SDCard_SPIM_SCB_MODE_I2C_CONST_CFG  && SDCard_SPIM_I2C_WAKE_ENABLE_CONST)
        SDCard_SPIM_I2CRestoreConfig();

    #elif (SDCard_SPIM_SCB_MODE_EZI2C_CONST_CFG && SDCard_SPIM_EZI2C_WAKE_ENABLE_CONST)
        SDCard_SPIM_EzI2CRestoreConfig();

    #elif (SDCard_SPIM_SCB_MODE_SPI_CONST_CFG && SDCard_SPIM_SPI_WAKE_ENABLE_CONST)
        SDCard_SPIM_SpiRestoreConfig();

    #elif (SDCard_SPIM_SCB_MODE_UART_CONST_CFG && SDCard_SPIM_UART_WAKE_ENABLE_CONST)
        SDCard_SPIM_UartRestoreConfig();

    #else

        if(0u != SDCard_SPIM_backup.enableState)
        {
            SDCard_SPIM_Enable();
        }

    #endif /* (SDCard_SPIM_I2C_WAKE_ENABLE_CONST) */

#endif /* (SDCard_SPIM_SCB_MODE_UNCONFIG_CONST_CFG) */
}


/* [] END OF FILE */
