/*******************************************************************************
* File Name: test.c  
* Version 2.20
*
* Description:
*  This file contains APIs to set up the Pins component for low power modes.
*
* Note:
*
********************************************************************************
* Copyright 2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "cytypes.h"
#include "test.h"

static test_BACKUP_STRUCT  test_backup = {0u, 0u, 0u};


/*******************************************************************************
* Function Name: test_Sleep
****************************************************************************//**
*
* \brief Stores the pin configuration and prepares the pin for entering chip 
*  deep-sleep/hibernate modes. This function must be called for SIO and USBIO
*  pins. It is not essential if using GPIO or GPIO_OVT pins.
*
* <b>Note</b> This function is available in PSoC 4 only.
*
* \return 
*  None 
*  
* \sideeffect
*  For SIO pins, this function configures the pin input threshold to CMOS and
*  drive level to Vddio. This is needed for SIO pins when in device 
*  deep-sleep/hibernate modes.
*
* \funcusage
*  \snippet test_SUT.c usage_test_Sleep_Wakeup
*******************************************************************************/
void test_Sleep(void)
{
    #if defined(test__PC)
        test_backup.pcState = test_PC;
    #else
        #if (CY_PSOC4_4200L)
            /* Save the regulator state and put the PHY into suspend mode */
            test_backup.usbState = test_CR1_REG;
            test_USB_POWER_REG |= test_USBIO_ENTER_SLEEP;
            test_CR1_REG &= test_USBIO_CR1_OFF;
        #endif
    #endif
    #if defined(CYIPBLOCK_m0s8ioss_VERSION) && defined(test__SIO)
        test_backup.sioState = test_SIO_REG;
        /* SIO requires unregulated output buffer and single ended input buffer */
        test_SIO_REG &= (uint32)(~test_SIO_LPM_MASK);
    #endif  
}


/*******************************************************************************
* Function Name: test_Wakeup
****************************************************************************//**
*
* \brief Restores the pin configuration that was saved during Pin_Sleep().
*
* For USBIO pins, the wakeup is only triggered for falling edge interrupts.
*
* <b>Note</b> This function is available in PSoC 4 only.
*
* \return 
*  None
*  
* \funcusage
*  Refer to test_Sleep() for an example usage.
*******************************************************************************/
void test_Wakeup(void)
{
    #if defined(test__PC)
        test_PC = test_backup.pcState;
    #else
        #if (CY_PSOC4_4200L)
            /* Restore the regulator state and come out of suspend mode */
            test_USB_POWER_REG &= test_USBIO_EXIT_SLEEP_PH1;
            test_CR1_REG = test_backup.usbState;
            test_USB_POWER_REG &= test_USBIO_EXIT_SLEEP_PH2;
        #endif
    #endif
    #if defined(CYIPBLOCK_m0s8ioss_VERSION) && defined(test__SIO)
        test_SIO_REG = test_backup.sioState;
    #endif
}


/* [] END OF FILE */
