//
// Graphics library by ladyada/adafruit with init code from Rossum, speed adapted and converted to C by C.K.
// MIT license

#include "ILI9341.h"

#define swap(a, b) { uint16_t t = a; a = b; b = t; }


// Font Indices
#define FONT_LENGTH		0
#define FONT_FIXED_WIDTH	2
#define FONT_HEIGHT		3
#define FONT_FIRST_CHAR		4
#define FONT_CHAR_COUNT		5
#define FONT_WIDTH_TABLE	6


#define ILI9341_INIT_START 0x01
#define ILI9341_PWRCTRL_A 0xCB
#define ILI9341_PWRCTRL_B 0xCF
#define ILI9341_TIMCTRL_A 0xE8
#define ILI9341_TIMCTRL_B 0xEA
#define ILI9341_PWRONSEQCTRL 0xED
#define ILI9341_PUMPRATIOCTRL 0xF7
#define ILI9341_PWRCTRL_1 0xC0
#define ILI9341_PWRCTRL_2 0xC1
#define ILI9341_VCOMCTRL_1 0xC5
#define ILI9341_FRAMERATE 0xB1
#define ILI9341_FUNCCTRL 0xB6
#define ILI9341_ENABLE3G 0xF2
#define ILI9341_COLORMOD 0x3a		// Pixel Format Set
#define ILI9341_GAMMASET 0x26   	// Gamma curve 3
#define ILI9341_POSGAMMACORRECT 0xE0
#define ILI9341_NEGGAMMACORRECT 0XE1
#define ILI9341_SLEEP_OUT 0x11
#define ILI9341_DISP_ON 0x29

#define ILI9341_SETORIENT 0x36
#define ILI9341_COLSET  0x2A
#define ILI9341_PAGESET  0x2B
#define ILI9341_RAMWR 0x2C

#define DELAY 0x80   // for defining delays in the command list

//
// gloabal variables
//


const uint8_t InitCmd[] = {  // Init Sequence for ILI9341 controller
  20,  // number of commands in the list
  ILI9341_INIT_START, DELAY, 100,	// 0 args, delay 100ms
  ILI9341_PWRCTRL_A, 5,			// 5 args, no delay
  0x39, 0x2C, 0x00, 0x34, 0x02,		// 3 args, no delay
  ILI9341_PWRCTRL_B, 3,
  0x00, 0XC1, 0X30,
  ILI9341_TIMCTRL_A, 3,
  0x85,0x00,0x78,
  ILI9341_TIMCTRL_B, 2,
  0x00, 0x00, 
  ILI9341_PWRONSEQCTRL, 4,
  0x64, 0x03, 0X12, 0X81,
  ILI9341_PUMPRATIOCTRL, 1,
  0x20, 
  ILI9341_PWRCTRL_1, 1,
  0x23,
  ILI9341_PWRCTRL_2, 1,
  0x10,
  ILI9341_VCOMCTRL_1, 2,
  0x2B, 0x2B,
  ILI9341_SETORIENT, 1,   // dummy set oientation
  0xE8, 
  ILI9341_FRAMERATE, 2,
  0x00, 0x1B,
  ILI9341_FUNCCTRL, 2,
  0x0A, 0x02,
  ILI9341_ENABLE3G, 1,
  0x02,
  ILI9341_COLORMOD, 1,
  0x05,
  ILI9341_GAMMASET , 1,
  0x01, 
  ILI9341_POSGAMMACORRECT, 15,
  0x0F, 0x31, 0x2B, 0x0C, 0x0E, 0x08, 0x4E, 0xF1, 0x37, 0x07, 0x10, 0x03, 0x0E, 0x09, 0x00,
  ILI9341_NEGGAMMACORRECT, 15,
  0x00, 0x0E, 0x14, 0x03, 0x11, 0x07, 0x31, 0xC1, 0x48, 0x08, 0x0F, 0x0C, 0x31, 0x36, 0x0F,
  ILI9341_SLEEP_OUT, DELAY, 120, 
  ILI9341_DISP_ON, 0
};

  uint8_t madctl;
  uint16_t _backgroundColor;

  uint16_t _width;
  uint16_t _height;

  // Arduino GLCD font
  Font_t  Font;

//
// Implementation
//

// SPI Interface
inline void writecommand(uint8_t cmd)
{ 
    D_C_Write(0);   
    CS_Write(0);
    SPIM_WriteTxData(cmd);
    CS_Write(1);
}  

inline void writedata(uint8_t Data)
{ 
    D_C_Write(1);
    CS_Write(0);
    SPIM_WriteTxData(Data); 
    CS_Write(1);
}  


// Companion code to the above tables. Reads and issues
// a series of LCD commands stored in const byte array.
void ILI9341_commandList(uint8_t const *addr) {
    uint8_t numCommands, numArgs;
    uint16_t ms;

    numCommands = *(addr++); // Number of commands to follow
    while(numCommands--) { // For each command...
        writecommand(*(addr++)); // Read, issue command
        numArgs = *(addr++); // Number of args to follow
        ms = numArgs & DELAY; // If hibit set, delay follows args
        numArgs &= ~DELAY; // Mask out delay bit
        while(numArgs--) { // For each argument...
            writedata(*(addr++)); // Read, issue argument
        }
        if (ms) {
            ms = *(addr++); // Read post-command delay time (ms)
                CyDelay(ms); // Delay in ms
        }
    }
}

void ILI9341_initR(const uint16_t width, const uint16_t height, uint8_t orient) {
  // SPI Interface has to be setup elsewhere
  // Is Display Reset required?

  // init display
  ILI9341_commandList(InitCmd);
  ILI9341_setRotation(orient);
  madctl = orient;
  _width = width;
  _height = height;
}

uint16_t ILI9341_width(void) {return(_width);};

uint16_t ILI9341_height(void) {return(_height);};

uint16_t ILI9341_backgroundColor(void) { return (_backgroundColor); };


void ILI9341_setAddrWindow(const uint16_t x0, uint16_t y0, const uint16_t x1, const uint16_t y1)
{
  writecommand(ILI9341_COLSET);  // column addr set
  writedata(x0 >> 8);
  writedata(x0);   // XSTART 
  writedata(x1 >> 8);
  writedata(x1);   // XEND

  writecommand(ILI9341_PAGESET);  // row addr set
  writedata(y0 >> 8);
  writedata(y0);    // YSTART
  writedata(y1 >> 8);
  writedata(y1);    // YEND

  writecommand(ILI9341_RAMWR);
}


void ILI9341_setRotation(uint8_t orient)
{
  writecommand(ILI9341_SETORIENT);
  switch (orient)
  {
    case 0: writedata(0x48);
            break;
    case 1: writedata(0x28);
            break;
    case 2: writedata(0x88);
            break;
    case 3: writedata(0xE8);
            break;
   }
   madctl=orient;
}

uint8_t ILI9341_getRotation() {
  return madctl;
}


void ILI9341_setBackgroundColor(const uint16_t color)
{
  _backgroundColor=color;
}

void ILI9341_fillScreen(const uint16_t color) {

  ILI9341_setAddrWindow(0, 0, _width-1, _height-1);

  //writedata(color >> 8);
  //writedata(color);
  D_C_Write(1);
  CS_Write(0);
  uint16_t x;
  uint16_t y;
  for (x=0; x < _width; x++) {
    for (y=0; y < _height; y++) {
      SPIM_WriteTxData(color >> 8); 
      SPIM_WriteTxData(color);  
    }
  }
  CS_Write(1);
}

void ILI9341_miscScreen(uint16_t color) {

  ILI9341_setAddrWindow(0, 0, _width-1, _height-1);

  // setup for data
  D_C_Write(1);
  CS_Write(0);
  uint16_t x;
  uint16_t y;
  for (x=0; x < _width; x++) {
    for (y=0; y < _height; y++) {
      SPIM_WriteTxData(color >> 8);    
      SPIM_WriteTxData(color);
      color++;
    }
  }
  CS_Write(1);
}

void ILI9341_drawPixel(const uint16_t x, const uint16_t y,const uint16_t color) {

  if ((x >= _width) || (y >= _height)) return;

  ILI9341_setAddrWindow(x,y,x+1,y+1);
  writedata(color >> 8);
  writedata(color);
}

// fill a circle
void ILI9341_fillCircle(const uint16_t x0, const uint16_t y0, const uint16_t r, const uint16_t color) {
  int16_t f = 1 - r;
  int16_t ddF_x = 1;
  int16_t ddF_y = -2 * r;
  int16_t x = 0;
  int16_t y = r;

  ILI9341_drawVerticalLine(x0, y0-r, 2*r+1, color);

  while (x<y) {
    if (f >= 0) {
      y--;
      ddF_y += 2;
      f += ddF_y;
    }
    x++;
    ddF_x += 2;
    f += ddF_x;
  
    ILI9341_drawVerticalLine(x0+x, y0-y, 2*y+1, color);
    ILI9341_drawVerticalLine(x0-x, y0-y, 2*y+1, color);
    ILI9341_drawVerticalLine(x0+y, y0-x, 2*x+1, color);
    ILI9341_drawVerticalLine(x0-y, y0-x, 2*x+1, color);
  }
}

// draw a circle outline
void ILI9341_drawCircle(const uint16_t x0, const uint16_t y0, const uint16_t r, 
			const uint16_t color) {
  int16_t f = 1 - r;
  int16_t ddF_x = 1;
  int16_t ddF_y = -2 * r;
  int16_t x = 0;
  int16_t y = r;

  ILI9341_drawPixel(x0, y0+r, color);
  ILI9341_drawPixel(x0, y0-r, color);
  ILI9341_drawPixel(x0+r, y0, color);
  ILI9341_drawPixel(x0-r, y0, color);

  while (x<y) {
    if (f >= 0) {
      y--;
      ddF_y += 2;
      f += ddF_y;
    }
    x++;
    ddF_x += 2;
    f += ddF_x;
  
    ILI9341_drawPixel(x0 + x, y0 + y, color);
    ILI9341_drawPixel(x0 - x, y0 + y, color);
    ILI9341_drawPixel(x0 + x, y0 - y, color);
    ILI9341_drawPixel(x0 - x, y0 - y, color);
    
    ILI9341_drawPixel(x0 + y, y0 + x, color);
    ILI9341_drawPixel(x0 - y, y0 + x, color);
    ILI9341_drawPixel(x0 + y, y0 - x, color);
    ILI9341_drawPixel(x0 - y, y0 - x, color);
  }
}


// draw a rectangle
void ILI9341_drawRect(const uint16_t x, const uint16_t y, const uint16_t w, const uint16_t h, 
		      const uint16_t color) {
  // smarter version
  ILI9341_drawHorizontalLine(x, y, w, color);
  ILI9341_drawHorizontalLine(x, y+h-1, w, color);
  ILI9341_drawVerticalLine(x, y, h, color);
  ILI9341_drawVerticalLine(x+w-1, y, h, color);
}

void ILI9341_fillRect(uint16_t x, uint16_t y, const uint16_t w, const uint16_t h, 
		      const uint16_t color) {
  // smarter version

  ILI9341_setAddrWindow(x, y, x+w-1, y+h-1);

  // setup for data
  D_C_Write(1);
  CS_Write(0);
  // display controller writes first in X and increments Y after window width
  for (y=0; y < h; y++) {
    for (x=0; x < w; x++) {
      SPIM_WriteTxData(color >> 8);    
      SPIM_WriteTxData(color);    
    }
  }
  CS_Write(1);
}

void ILI9341_drawVerticalLine(const uint16_t x, const uint16_t y, uint16_t length, const uint16_t color)
{
  if (x >= _width) return;
  if (y+length >= _height) length = _height-y-1;

  ILI9341_drawFastLine(x,y,length,color,1);
}

void ILI9341_drawHorizontalLine(const uint16_t x, const uint16_t y, uint16_t length, const uint16_t color)
{
  if (y >= _height) return;
  if (x+length >= _width) length = _width-x-1;

  ILI9341_drawFastLine(x,y,length,color,0);
}

void ILI9341_drawFastLine(const uint16_t x, const uint16_t y, uint16_t length, 
			  const uint16_t color, const uint8_t rotflag)
{
  if (rotflag) {
    ILI9341_setAddrWindow(x, y, x, y+length);
  } else {
    ILI9341_setAddrWindow(x, y, x+length, y+1);
  }
  // setup for data
  D_C_Write(1);
  CS_Write(0);
  while (length--) {
    SPIM_WriteTxData(color >> 8);    
    SPIM_WriteTxData(color);    
  }
  CS_Write(1);
}


// bresenham's algorithm - thx wikpedia
void ILI9341_drawLine(int16_t x0, int16_t y0, int16_t x1, int16_t y1, const uint16_t color)
{
  uint16_t steep = abs(y1 - y0) > abs(x1 - x0);
  if (steep) {
    swap(x0, y0);
    swap(x1, y1);
  }

  if (x0 > x1) {
    swap(x0, x1);
    swap(y0, y1);
  }

  uint16_t dx, dy;
  dx = x1 - x0;
  dy = abs(y1 - y0);

  int16_t err = dx / 2;
  int16_t ystep;

  if (y0 < y1) {
    ystep = 1;
  } else {
    ystep = -1;}

  for (; x0<=x1; x0++) {
    if (steep) {
      ILI9341_drawPixel(y0, x0, color);
    } else {
      ILI9341_drawPixel(x0, y0, color);
    }
    err -= dy;
    if (err < 0) {
      y0 += ystep;
      err += dx;
    }
  }
}


/**
 * Draw a rectangle with rounder corners
 *
 * @param x the x coordinate of the upper left corner of the rectangle
 * @param y the y coordinate of the upper left corner of the rectangle
 * @param width width of the rectangle
 * @param height height of the rectangle
 * @param radius ????
 * @param 16-bit color default BLACK 
 *
 * Draws a rectangle the same as DrawRect() but with rounded corners.
 * Radius is a value from 1 to half the height or width of the rectangle.
 * (what does that really mean?????)
 * Which is it, or is it a the smaller of the two?
 * FIXME FIXME need more description here.
 *
 * @see DrawRect()
 */

void ILI9341_drawRoundRect(const uint16_t x, const uint16_t y, const uint16_t width, const uint16_t height, 
                           const uint16_t radius, const uint16_t color) 
{
  int16_t tSwitch; 
  uint16_t x1 = 0, y1 = radius;
  tSwitch = 3 - 2 * radius;
	
  while (x1 <= y1) {
    ILI9341_drawPixel(x+radius - x1, y+radius - y1, color);
    ILI9341_drawPixel(x+radius - y1, y+radius - x1, color);

    ILI9341_drawPixel(x+width-radius + x1, y+radius - y1, color);
    ILI9341_drawPixel(x+width-radius + y1, y+radius - x1, color);
	    
    ILI9341_drawPixel(x+width-radius + x1, y+height-radius + y1, color);
    ILI9341_drawPixel(x+width-radius + y1, y+height-radius + x1, color);

    ILI9341_drawPixel(x+radius - x1, y+height-radius + y1, color);
    ILI9341_drawPixel(x+radius - y1, y+height-radius + x1, color);

    if (tSwitch < 0) {
    	tSwitch += (4 * x1 + 6);
    } else {
      tSwitch += (4 * (x1 - y1) + 10);
      y1--;
    }
    x1++;
  }
	  	
  ILI9341_drawHorizontalLine(x+radius, y, width-(2*radius), color);		// top
  ILI9341_drawHorizontalLine(x+radius, y+height, width-(2*radius), color);	// bottom
  ILI9341_drawVerticalLine(x, y+radius, height-(2*radius), color);		// left
  ILI9341_drawVerticalLine(x+width, y+radius, height-(2*radius), color);	// right
}

uint8_t ILI9341_isFixedWidthFont(void)
{
  return( (*(Font+FONT_LENGTH) == 0) && (*(Font+FONT_LENGTH+1) == 0) );
}

void ILI9341_selectFont(const Font_t font)
{
	Font = font;
}

uint8_t ILI9341_getFontHeight(void)
{
  return( *(Font+FONT_HEIGHT) );
}

uint16_t ILI9341_putChar(uint16_t x, const uint16_t y, uint8_t c, const uint16_t color)
// returns x position after printed character
// function is a little bit speed optimized, with AT328p 16MHz: 
// 0.61ms for 5x7 character
// 1.26ms for a 7x15 character 
// 3.55ms for a 10x24 character
{
  if (Font == 0)
    return x; // no font selected

  /*
  * check for special character processing
  */
  if (c < 0x20) {  // special char, do nothing
    return x;
  }
	   
  uint8_t fontWidth = 0;
  uint8_t FontHeight = *(Font+FONT_HEIGHT);
  uint8_t bytes = (FontHeight+7)/8; /* calculates FontHeight in rounded up bytes */

  uint8_t firstChar = *(Font+FONT_FIRST_CHAR);
  uint8_t charCount = *(Font+FONT_CHAR_COUNT);
	
  uint16_t index = 0;

  if (c < firstChar || c >= (firstChar+charCount)) {
    return x; // invalid char
  }
  c-= firstChar;

  if ( ILI9341_isFixedWidthFont() ) {
    fontWidth = *(Font+FONT_FIXED_WIDTH); 
    index = c*bytes*fontWidth+FONT_WIDTH_TABLE;
  }
  else  {
    // variable width font, read width data, to get the index
    /*
     * Because there is no table for the offset of where the data
     * for each character glyph starts, run the table and add up all the
     * widths of all the characters prior to the character we
     * need to locate.
     */
    uint8_t i;
    for (i=0; i<c; i++) {  
      index += *(Font+FONT_WIDTH_TABLE+i);
    }
    /*
     * Calculate the offset of where the font data
     * for our character starts.
     * The index value from above has to be adjusted because
     * there is potentialy more than 1 byte per column in the glyph,
     * when the characgter is taller than 8 bits.
     * To account for this, index has to be multiplied
     * by the FontHeight in bytes because there is one byte of font
     * data for each vertical 8 pixels.
     * The index is then adjusted to skip over the font fontWidth data
     * and the font header information.
     */
     index = index*bytes+charCount+FONT_WIDTH_TABLE;
     /*
      * Finally, fetch the fontWidth of our character
      */
     fontWidth = *(Font+FONT_WIDTH_TABLE+c);
  }  // end else

  // if character does not fit to the screen, write it at the end (no line wrap)
  if (x + fontWidth >= _width)  {
    x=_width-fontWidth;
  }

  // last but not least, draw the character
  ILI9341_setAddrWindow(x, y, x+fontWidth-1, y+FontHeight-1);

  // setup display for data transmission
  D_C_Write(1);
  CS_Write(0);

  /*
   * Need to write to Display Memory in X direction
   * Y automatically increments due to set window size
   * Because GLCD fonts are organized in Y direction, need some ugly 
   * Font Data accesses to map the Font to the display memory
   * GLCD has stored the font in the following way:
   * Each byte is one X-step and eight Y-Steps. First X/Y pixel is given by the LSB
   * After all Bytes of one X line (font width) the next Y+8 pixel are coming
   * if less than 8-bit are used in a byte the MSB's are not used (at least for size 7)
   */
  uint8_t i;
  for (i=0; i<bytes; i++)	/* each vertical byte */
  { 
    uint16_t page = i*fontWidth; // page must be 16 bit to prevent overflow		
    uint8_t bitpnt=0x01;    // pointer to the bit to be drawn
    uint8_t nr_of_pixel=8;  // initial value for a byte
    const uint8_t* tmpFontPnt=Font+index+page;

    /* is it last byte of multibyte tall font? or a very small font? */
    uint8_t lastByte=( FontHeight > 8 && FontHeight < (i+1)*8 ) || (FontHeight < 8);
    if ( lastByte )
    /* is it last byte of multibyte tall font? or a very small font? */
    {
      nr_of_pixel=FontHeight-i*8;
      if (FontHeight<8)  // Workaround: bug in font-data for small fonts...
         bitpnt <<= (i+1)*8-FontHeight-1;        // remove unused bits
       else
         bitpnt <<= (i+1)*8-FontHeight;        
    }
    uint8_t p;
    for (p=0; p<nr_of_pixel; p++)  {   // p increments in y direction
      //uint8_t yp = i*8+p;
      uint8_t j;
      for (j=0; j<fontWidth; j++)  {   // write in x-direction
         uint8_t data = *(tmpFontPnt+j);
         if (data & bitpnt)  {
           SPIM_WriteTxData(color >> 8);    
           SPIM_WriteTxData(color);    
           //drawPixel(x+j, y+yp, color);
         }
         else  {
           SPIM_WriteTxData(_backgroundColor >> 8);    
           SPIM_WriteTxData(_backgroundColor);    
           //drawPixel(x+j, y+yp, _backgroundColor);
         }
       }  // for j
       bitpnt <<= 1;
     }  // end  for p
  }  // end for i
  CS_Write(1);
  return (x+fontWidth); // valid char, return new x position
}

uint16_t ILI9341_putS(uint16_t x, const uint16_t y, const char *string, const uint16_t color)
{
  uint8_t first=1;
  uint8_t fontHeight = *(Font+FONT_HEIGHT);
  char * str=(char *)string;

  while(*str)  {
    if (!first)  {
      // space between characters
      ILI9341_drawVerticalLine(x,y,fontHeight, _backgroundColor);
      x++;
    }
    x=ILI9341_putChar(x, y, (uint8_t)*str, color);
    str++;
    first=0;
  }
  return(x);
}



/**
 * Returns the pixel width of a character
 *
 * @param c character to be sized
 *
 * @return The width in pixels of the given character
 * including any inter-character gap pixels following the character when the character is
 * rendered on the display.
 *
 * @note The font for the character is the most recently selected font.
 *
 * @see StringWidth()
 * @see StringWidth_P()
 */

uint8_t ILI9341_charWidth(uint8_t c)
{
  uint8_t width = 0;
	
  if  ( ILI9341_isFixedWidthFont() ) {
    width = *(Font+FONT_FIXED_WIDTH)+1;  // there is 1 pixel pad here
  } 
  else  { 
    // variable width font 
    uint8_t firstChar = *(Font+FONT_FIRST_CHAR);
    uint8_t charCount = *(Font+FONT_CHAR_COUNT);
	
    // read width data
    if (c >= firstChar && c < (firstChar+charCount)) {
      c -= firstChar;
      width = *(Font+FONT_WIDTH_TABLE+c)+1;
    }
  }	
  return width;
}

/**
 * Returns the pixel width of a string
 *
 * @param str pointer to string stored in RAM
 *
 * @return the width in pixels of the sum of all the characters in the
 * the string pointed to by str. 
 *
 * @see CharWidth()
 * @see StringWidth_P()
 */

uint16_t ILI9341_stringWidth(const char* str)
{
  uint16_t width = 0;
	
  while(*str != 0) {
    width += ILI9341_charWidth(*str++);
  }	
  return width;
}


// end of file
