// Graphics library for ILI9341 LCD Displays 
// MIT license

#ifndef ILI9341_H
#define ILI9341_H

#include <stdlib.h>
#include <project.h>	//   uint_t !

// color definition for RGB (5,6,5 bit)
#define	BLACK           (uint16_t)(~0x0000)
#define	BLUE            (uint16_t)(~0x001F)
#define LIGHTBLUE	    (uint16_t)(~0xE7DF)
#define	RED             (uint16_t)(~0xF800)
#define LIGHTRED        (uint16_t)(~0xFF1C)
#define	GREEN           (uint16_t)(~0x07E0)
#define CYAN            (uint16_t)(~0x07FF)
#define MAGENTA         (uint16_t)(~0xF81F)
#define YELLOW          (uint16_t)(~0xFFE0)
#define WHITE           (uint16_t)(~0xFFFF)



// the following returns true if the given font is fixed width
// zero length is flag indicating fixed width font (array does not contain width data entries)

typedef const uint8_t* Font_t;
typedef uint8_t (*FontCallback)(Font_t);


  inline void writecommand(const uint8_t);
  inline void writedata(const uint8_t);



  //void initB(void);
  void ILI9341_initR(const uint16_t width, const uint16_t height, uint8_t orient);

  // get actual display width
  uint16_t ILI9341_width(void);
  
  // get actual display height
  uint16_t ILI9341_height(void);

  // drawing primitives!
  void ILI9341_setAddrWindow(const uint16_t x0, const uint16_t y0, const uint16_t x1, const uint16_t y1);
  void ILI9341_setBackgroundColor(const uint16_t color);
  uint16_t ILI9341_backgroundColor(void);

  void ILI9341_fillScreen(const uint16_t color);
  void ILI9341_miscScreen(uint16_t color);
  void ILI9341_drawPixel(const uint16_t x, const uint16_t y, const uint16_t color);

  void ILI9341_drawLine(int16_t x, int16_t y, int16_t x1, int16_t y1, uint16_t color);
  void ILI9341_drawVerticalLine(const uint16_t x0, const uint16_t y0, uint16_t length, const uint16_t color);
  void ILI9341_drawHorizontalLine(const uint16_t x0, const uint16_t y0,uint16_t length, const  uint16_t color);
  void ILI9341_drawFastLine(const uint16_t x0, const uint16_t y0, uint16_t l,const uint16_t color, const uint8_t flag);

  void ILI9341_drawRect(const uint16_t x, const uint16_t y, const uint16_t w, const uint16_t h, const uint16_t color);
  void ILI9341_fillRect(uint16_t x, uint16_t y, const uint16_t w, const uint16_t h, const uint16_t color);
  void ILI9341_drawRoundRect(const uint16_t x, const uint16_t y, const uint16_t width, const uint16_t height, 
                           const uint16_t radius, const uint16_t color);

  void ILI9341_drawCircle(const uint16_t x0, const uint16_t y0, const uint16_t r, const uint16_t color);
  void ILI9341_fillCircle(uint16_t x0, uint16_t y0, uint16_t r, uint16_t color);

// for arduino GLCD

  void ILI9341_selectFont(const Font_t font);
  uint8_t ILI9341_getFontHeight(void);
  uint16_t ILI9341_putChar(uint16_t x, const uint16_t y, uint8_t c, const uint16_t color);
  uint16_t ILI9341_putS(uint16_t x, const uint16_t y, const char *str, const uint16_t color);

  uint8_t ILI9341_charWidth(uint8_t c);
  uint16_t ILI9341_stringWidth(const char* str);

// end for arduino GLCD

  // rotation 0 : 240x320
  // rotation 2 : 240x320
  // rotation 1 : 320x240
  // rotation 3 : 320x240
  void ILI9341_setRotation(const uint8_t);
  uint8_t ILI9341_getRotation(void);

  void ILI9341_dummyclock(void);

#endif
// end of file
